package com.smthng.quantumbags;

import net.minecraft.ChatFormatting;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.InteractionResultHolder;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.SlotAccess;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.ClickAction;
import net.minecraft.world.inventory.Slot;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.TooltipFlag;
import net.minecraft.world.level.Level;
import net.minecraftforge.common.ForgeConfigSpec;
import org.jetbrains.annotations.NotNull;

import javax.annotation.Nullable;
import java.util.List;

public class QuantumBag extends Item {
    ForgeConfigSpec.ConfigValue<Integer> size;

    public QuantumBag(Properties pProperties, ForgeConfigSpec.ConfigValue<Integer> size) {
        super(pProperties);
        this.size = size;
    }

    @Override
    public boolean overrideOtherStackedOnMe(ItemStack pStack, ItemStack pOther, Slot pSlot, ClickAction pAction, Player pPlayer, SlotAccess pAccess) {
        boolean canBeStored = !(pOther.getItem().equals(Items.AIR) || pOther.getItem().equals(this));
        if (pAction.equals(ClickAction.PRIMARY)) { // left click on bag
            if (canBeStored && matchesStored(pStack, pOther)) {
                addItems(pStack, pOther);
                return true;
            }
        } else { // right click on bag
            if (canBeStored) {
                if (matchesStored(pStack, pOther)) {
                    addOne(pStack, pOther);
                }
            } else {
                rotateMode(pStack);
            }
            return true;
        }
        return false;
    }

    @Override
    public void inventoryTick(@NotNull ItemStack pStack, @NotNull Level pLevel, @NotNull Entity pEntity, int pSlotId, boolean pIsSelected) {
        if (pEntity instanceof Player pPlayer) {
            if (mode(pStack) == 1) { // fill inv from bag
                ItemStack stored = getStoredStack(pStack);
                for (ItemStack itemStack : pPlayer.getInventory().items) {
                    if (stored.getCount() > 0) {
                        if (itemStackSame(stored, itemStack)) {
                            int removeAmount = Math.min(itemStack.getMaxStackSize() - itemStack.getCount(), storedItemCount(pStack));
                            stored.setCount(stored.getCount() - removeAmount);

                            itemStack.setCount(itemStack.getCount() + removeAmount);
                        }
                    } else {
                        break;
                    }
                }
                setStoredStack(pStack, stored);
            } else if (mode(pStack) == 2) { // suck items into bag
                ItemStack stored = getStoredStack(pStack);
                for (ItemStack itemStack : pPlayer.getInventory().items) {
                    if (stored.getCount() < size.get()) {
                        if (itemStackSame(stored, itemStack)) {
                            int removeAmount = Math.min(size.get() - stored.getCount(), itemStack.getCount());
                            stored.setCount(stored.getCount() + removeAmount);

                            itemStack.setCount(itemStack.getCount() - removeAmount);
                        }
                    } else {
                        break;
                    }
                }
                setStoredStack(pStack, stored);
            }
        }
    }

    public @NotNull InteractionResultHolder<ItemStack> use(@NotNull Level pLevel, @NotNull Player pPlayer, @NotNull InteractionHand pUsedHand) {
        ItemStack itemStack = pPlayer.getItemInHand(pUsedHand);
        rotateMode(itemStack);
        return InteractionResultHolder.success(itemStack);
    }

    public boolean onDroppedByPlayer(ItemStack item, Player player)
    {
        ItemStack stored = getStoredStack(item);
        if (stored.getCount() > 0) {
            ItemStack drop = stored.copy();
            stored.setCount(stored.getCount() - 1);
            setStoredStack(item, stored);
            drop.setCount(1);
            player.level().addFreshEntity(player.drop(drop, false, true));
            return false;
        } else {
            return true;
        }
    }

    private void addItems(@NotNull ItemStack bag, @NotNull ItemStack itemStack) {
        int removeAmount = Math.min(size.get() - storedItemCount(bag), itemStack.getCount());
        if (removeAmount > 0) {
            ItemStack stored = itemStack.copy();
            stored.setCount(removeAmount + storedItemCount(bag));
            setStoredStack(bag, stored);
            itemStack.setCount(itemStack.getCount() - removeAmount);
        }
    }

    private void addOne(@NotNull ItemStack bag, @NotNull ItemStack itemStack) {
        int removeAmount = Math.min(size.get() - storedItemCount(bag), 1);
        if (removeAmount > 0) {
            ItemStack stored = itemStack.copy();
            stored.setCount(removeAmount + storedItemCount(bag));
            setStoredStack(bag, stored);
            itemStack.setCount(itemStack.getCount() - removeAmount);
        }
    }

    private static boolean matchesStored(ItemStack bag, @NotNull ItemStack itemStack) {
        ItemStack stored = getStoredStack(bag).copy();
        stored.setCount(1);
        ItemStack compared = itemStack.copy();
        compared.setCount(1);
        return stored.serializeNBT().equals(compared.serializeNBT()) || !hasStoredItem(bag) || stored.getItem().equals(Items.AIR);
    }

    private static boolean itemStackSame(ItemStack stack1, ItemStack stack2) {
        ItemStack s1 = stack1.copy();
        s1.setCount(1);
        ItemStack s2 = stack2.copy();
        s2.setCount(1);
        return s1.serializeNBT().equals(s2.serializeNBT());
    }

    private static int storedItemCount(ItemStack bag) {
        return getStoredStack(bag).getCount();
    }

    private static @NotNull ItemStack getStoredStack(@NotNull ItemStack bag) {
        CompoundTag compoundTag = bag.getOrCreateTag();
        ItemStack stored = ItemStack.of(compoundTag.getCompound("stored_item_stack"));
        stored.setCount(compoundTag.getInt("stored_item_count"));
        return stored;
    }

    private static void setStoredStack(@NotNull ItemStack bag, @NotNull ItemStack itemStack) {
        CompoundTag compoundTag = bag.getOrCreateTag();
        String s1 = String.valueOf(itemStack.serializeNBT());
        compoundTag.put("stored_item_stack", itemStack.serializeNBT());
        compoundTag.putInt("stored_item_count", itemStack.getCount());
        bag.setTag(compoundTag);
    }

    private static boolean hasStoredItem(@NotNull ItemStack bag) {
        return bag.getOrCreateTag().contains("stored_item_stack");
    }

    private static int mode(@NotNull ItemStack bag) {
        CompoundTag compoundTag = bag.getOrCreateTag();
        return compoundTag.getInt("CustomModelData");
    }

    private static void rotateMode(@NotNull ItemStack bag) {
        CompoundTag compoundTag = bag.getOrCreateTag();
        compoundTag.putInt("CustomModelData", (compoundTag.getInt("CustomModelData") + 1) % 3);
    }

    @Override
    public ItemStack getDefaultInstance() {
        ItemStack itemStack = new ItemStack(this);
        CompoundTag compoundTag = itemStack.getOrCreateTag();
        compoundTag.putInt("CustomModelData", 0);
        itemStack.setTag(compoundTag);
        return itemStack;
    }

    @Override
    public void appendHoverText(@NotNull ItemStack pStack, @Nullable Level pLevel, @NotNull List<Component> pTooltipComponents, @NotNull TooltipFlag pIsAdvanced) {
        if (hasStoredItem(pStack)) {
            ItemStack stored = getStoredStack(pStack);
            if (!stored.getItem().equals(Items.AIR)) {
                pTooltipComponents.add(Component.literal(stored.getCount() + " ").withStyle(ChatFormatting.GRAY).append(Component.translatable(stored.getDescriptionId()).withStyle(ChatFormatting.WHITE)));
                //pTooltipComponents.add(Component.translatable(stored.getDescriptionId()).append(Component.literal(" : " + stored.getCount())).withStyle(ChatFormatting.DARK_BLUE));
            }
        }
        super.appendHoverText(pStack, pLevel, pTooltipComponents, pIsAdvanced);
    }
}
